from libc.stdint cimport *
from cython.operator import dereference
import binascii

cimport _SpookyV2

# Fast non-incremental hash functions

cpdef hash32(bytes message, uint32_t seed=0):
  return _SpookyV2.Hash32(message, len(message), seed)

cpdef hash64(bytes message, uint64_t seed=0):
  return _SpookyV2.Hash64(message, len(message), seed)

cpdef hash128(bytes message, uint64_t seed1=0, uint64_t seed2=0):
  cdef char digest[16]
  (<uint64_t*>&digest[0])[0] = seed1
  (<uint64_t*>&digest[8])[0] = seed2
  _SpookyV2.Hash128(message, len(message), <uint64_t*>(digest), <uint64_t*>(digest+8))
  return digest[:16]

cpdef hash128_long(bytes message, uint64_t seed1=0, uint64_t seed2=0):
  return long(binascii.hexlify(hash128(message, seed1, seed2)), 16)

# Incremental hashlib-style hash builders.  These bench almost 2x slower than the one-shot
# functions above.

cdef class _Hash:
  cdef _SpookyV2.SpookyHash __hash
  cdef int __digest_size

  def __cinit__(_Hash self, *args, **kwargs):
    pass # def needed so that __cinit__ dispatch is fast for derived classes

  cpdef update(_Hash self, bytes message):
    self.__hash.Update(message, len(message))
    return self

  cpdef bytes digest(_Hash self):
    cdef uint64_t digest[2]
    self.__hash.Final(&digest[0], &digest[1])
    return (<char*>digest)[:self.__digest_size]

  cpdef str hexdigest(_Hash self):
    return binascii.hexlify(self.digest())

  cpdef copy(_Hash self):
    raise NotImplementedError()

  property digest_size:
    def __get__(_Hash self):
      return self.__digest_size

  property block_size:
    def __get__(_Hash self):
      return 16

cdef class Hash32(_Hash):
  def __cinit__(Hash32 self, bytes message=None, uint32_t seed=0):
    self.__digest_size = 4
    self.__hash.Init(seed, seed)
    if message:
      self.__hash.Update(message, len(message))

  def __long__(Hash32 self):
    cdef uint64_t hash1, hash2
    self.__hash.Final(&hash1, &hash2)
    return <uint32_t>hash1

  cpdef copy(Hash32 self):
    copy = Hash32()
    copy.__hash = self.__hash
    return copy

cdef class Hash64(_Hash):
  def __cinit__(Hash64 self, bytes message=None, uint64_t seed=0):
    self.__digest_size = 8
    self.__hash.Init(seed, seed)
    if message:
      self.__hash.Update(message, len(message))

  def __long__(Hash64 self):
    cdef uint64_t hash1, hash2
    self.__hash.Final(&hash1, &hash2)
    return hash1

  cpdef copy(Hash64 self):
    copy = Hash64()
    copy.__hash = self.__hash
    return copy

cdef class Hash128(_Hash):
  def __cinit__(Hash128 self, bytes message=None, uint64_t seed1=0, uint64_t seed2=0):
    self.__digest_size = 16
    self.__hash.Init(seed1, seed2)
    if message:
      self.__hash.Update(message, len(message))

  def __long__(Hash128 self):
    return long(self.hexdigest(), 16)

  cpdef copy(Hash128 self):
    copy = Hash128()
    copy.__hash = self.__hash
    return copy





from libc.stdint cimport *

cdef extern from 'SpookyV2.h':
  cdef cppclass SpookyHash:
    void Init(uint64_t seed1, uint64_t seed2)
    void Update(const char *message, size_t length)
    void Final(uint64_t *hash1, uint64_t *hash2)

cdef extern from 'SpookyV2.h' namespace 'SpookyHash':
    void Hash128(const char *message, size_t length, uint64_t *hash1, uint64_t *hash2)
    uint64_t Hash64(const char *message, size_t length, uint64_t seed)
    uint32_t Hash32(const char *message, size_t length, uint32_t seed)



#!/usr/bin/env python
from ez_setup import use_setuptools
use_setuptools()

import sys
from setuptools import setup, Extension

try:
    from Cython.Distutils import build_ext
    from Cython.Build import cythonize
except ImportError:
    print("Could not import Cython. Install `cython` and rerun.")
    sys.exit(1)

setup(
  name='spooky_hash',
  version='1.0',

  description='Python wrapper for SpookyHash V2',
  author='Sergey Grankin',
  url='https://github.com/sgrankin/spooky_hash',
  long_description = open('README.md').read(),
  classifiers = [
    'Development Status :: 4 - Beta',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Cython',
    'Topic :: Software Development :: Libraries',
    'Topic :: Software Development',
  ],

  setup_requires=['nose>=1.0'],
  include_package_data=True,
  ext_modules=cythonize(
    [
      Extension(
        'spooky_hash',
        ['src/spooky_hash.pyx', 'src/SpookyV2.cpp'],
        language='c++'),
    ],
  ),
)



from nose.tools import *

import spooky_hash
import binascii

def test_Hash32():
   h = spooky_hash.Hash32('hello', seed=0)
   copy = h.copy()
   h.update('world')
   eq_(4, h.digest_size)
   eq_('f47d8a47', h.hexdigest())
   eq_(binascii.unhexlify('f47d8a47'), h.digest())
   eq_(0x478a7df4, long(h))

   assert_not_equal(0x478a7df4, long(copy))
   copy.update('world')
   eq_(0x478a7df4, long(copy))

def test_Hash64():
   h = spooky_hash.Hash64('hello', seed=0)
   copy = h.copy()
   h.update('world')
   eq_(8, h.digest_size)
   eq_('f47d8a477ddd6b4f', h.hexdigest())
   eq_(binascii.unhexlify('f47d8a477ddd6b4f'), h.digest())
   eq_(0x4f6bdd7d478a7df4, long(h))

   assert_not_equal(0x4f6bdd7d478a7df4, long(copy))
   copy.update('world')
   eq_(0x4f6bdd7d478a7df4, long(copy))

def test_Hash128():
   h = spooky_hash.Hash128('hello', seed1=0, seed2=0)
   copy = h.copy()
   h.update('world')
   eq_(16, h.digest_size)
   eq_('f47d8a477ddd6b4fbf3b493f8396830d', h.hexdigest())
   eq_(binascii.unhexlify('f47d8a477ddd6b4fbf3b493f8396830d'), h.digest())
   eq_(0xf47d8a477ddd6b4fbf3b493f8396830d, long(h))

   assert_not_equal('f47d8a477ddd6b4fbf3b493f8396830d', copy.hexdigest())
   copy.update('world')
   eq_(0xf47d8a477ddd6b4fbf3b493f8396830d, long(copy))

def test_hash32():
   h = spooky_hash.hash32('hi', seed=0)
   eq_(0x9aa6d50f, h)

   h = spooky_hash.hash32('helloworld', seed=0)
   eq_(0x478a7df4, h)

def test_hash64():
   h = spooky_hash.hash64('helloworld', seed=0)
   eq_(0x4f6bdd7d478a7df4, h)

def test_hash128():
   h = spooky_hash.hash128('hi', seed1=0, seed2=0)
   eq_('\x0f\xd5\xa6\x9a\x2a\x77\x5a\xcd\x50\xed\xd1\xb6\xdc\xa8\x63\x1d', h)
   h = spooky_hash.hash128('helloworld')
   eq_(binascii.unhexlify('f47d8a477ddd6b4fbf3b493f8396830d'), h)

def test_hash128_long():
   eq_(0xf47d8a477ddd6b4fbf3b493f8396830d, spooky_hash.hash128_long('helloworld'))

